package gs.datagen.v0

/** Type class for data which can be generated.
  */
trait Generated[A]:
  def generate(): A

object Generated:

  /** Summon an instance of the [[Generated]] type class.
    *
    * @param G
    *   The type class instance.
    * @return
    *   The summoned instance of [[Generated]].
    */
  def apply[A](
    using
    G: Generated[A]
  ): Generated[A] = G

  /** Implementation of [[Generated]] based on an instance of [[Gen]].
    *
    * @param gen
    *   The underlying [[Gen]] used to produce values.
    */
  final class FromGenerator[A](gen: Gen[A]) extends Generated[A]:
    override def generate(): A = gen.gen()

  /** Used to produce instances of the `Generated` type class.
    *
    * {{{
    * case class MyThing(x: String, y: Int, z: UUID)
    *
    * given genMyThing: Generated[MyThing] =
    *   Gen.of {
    *     for
    *       x <- Gen.string.alpha(Size.fixed(5))
    *       y <- Gen.integer.positive()
    *       z <- Gen.uuid.random()
    *     yield
    *       MyThing(x, y, z)
    *   }
    * }}}
    */
  def of[A](g: Gen[A]): Generated[A] = new FromGenerator[A](g)

end Generated
